local addonName, farmerVars = ...

local currentVersion = 0110020

local rarityList = {
  [0] = 'Poor',
  'Common',
  'Uncommon',
  'Rare',
  'Epic',
  'Legendary',
  'Artifact',
  'Heirloom',
  'WoW Token'
}

local checkButtonList = {}
local sliderList = {}
local events = {}

farmerVars.rarityColors = {}
for i = 0, 8 do
  farmerVars.rarityColors[i] = {GetItemQualityColor(i)}
end

local farmerOptionsFrame = CreateFrame('Frame', 'farmerOptionsFrame', UIParent)
farmerOptionsFrame.name = 'Farmer'
InterfaceOptions_AddCategory(farmerOptionsFrame)

local function setDefaultPosition ()
  local frame = farmerVars.frame
  frame:ClearAllPoints()
  frame:SetPoint('BOTTOM', nil, 'CENTER', 0, 35)
end

local function storePosition (frame)
  local icon = GetItemIcon(114978)

  icon = ' |T' .. icon .. ':0|t '
  farmerOptions.anchor = {frame:GetPoint()}
  frame:EnableMouse(false)
  frame:SetMovable(false)
  frame:SetFading(true)
  frame:Clear()
  frame:AddMessage(icon)
  frame:StopMovingOrSizing()
  frame:SetScript('OnDragStart', nil)
  frame:SetScript('OnReceiveDrag', nil)
end

local function moveFrame ()
  local frame = farmerVars.frame
  local icon = GetItemIcon(114978)

  icon = ' |T' .. icon .. ':0|t '
  frame:RegisterForDrag('LeftButton')
  frame:SetFading(false)
  frame:Clear()
  frame:AddMessage(icon)
  frame:EnableMouse(true)
  frame:SetMovable(true)
  frame:SetScript('OnDragStart', function (self)
    if (self:IsMovable() == true) then
      self:StartMoving()
    end
  end)
  frame:SetScript('OnReceiveDrag', function (self)
    storePosition(self)
  end)
end

local function displayRarity (edit, rarity)
  local colorHex
  colorHex = farmerVars.rarityColors[rarity][4]
  edit:SetText('|c'.. colorHex .. rarityList[rarity])
  edit:SetCursorPosition(0)
end

local function setFontSize (size)
  farmerOptions.fontSize = size
  farmerVars.font:SetFont('Fonts\\FRIZQT__.TTF', size, 'thickoutline')
end

local function initPanel ()
  local anchor
  local checkButton
  local slider
  local edit
  local button

  checkButton = CreateFrame('CheckButton', 'fastLootCheckButton', farmerOptionsFrame, 'OptionsCheckButtonTemplate')
  checkButton:SetPoint('TOPLEFT', farmerOptionsFrame, 10, -8)
  fastLootCheckButtonText:SetText('enable fast autoloot')
  checkButtonList['fastLoot'] = checkButton
  anchor = checkButton

  checkButton = CreateFrame('CheckButton', 'reagentsCheckButton', farmerOptionsFrame, 'OptionsCheckButtonTemplate')
  checkButton:SetPoint('TOPLEFT', anchor, 'BOTTOMLEFT', 0, -10)
  reagentsCheckButtonText:SetText('show reagents')
  checkButtonList['reagents'] = checkButton
  anchor = checkButton

  checkButton = CreateFrame('CheckButton', 'currencyCheckButton', farmerOptionsFrame, 'OptionsCheckButtonTemplate')
  checkButton:SetPoint('TOPLEFT', anchor, 'BOTTOMLEFT', 0, -5)
  currencyCheckButtonText:SetText('show currencies')
  checkButtonList['currency'] = checkButton
  anchor = checkButton

  checkButton = CreateFrame('CheckButton', 'moneyCheckButton', farmerOptionsFrame, 'OptionsCheckButtonTemplate')
  checkButton:SetPoint('TOPLEFT', anchor, 'BOTTOMLEFT', 0, -5)
  moneyCheckButtonText:SetText('show money')
  checkButtonList['money'] = checkButton
  checkButton:SetScript('OnClick', function (self)
    if (self:GetChecked() == true) then
      farmerVars.moneyStamp = GetMoney()
    end
  end)
  anchor = checkButton

  checkButton = CreateFrame('CheckButton', 'specialCheckButton', farmerOptionsFrame, 'OptionsCheckButtonTemplate')
  checkButton:SetPoint('TOPLEFT', anchor, 'BOTTOMLEFT', 0, -5)
  specialCheckButtonText:SetText('always show farming items')
  checkButtonList['special'] = checkButton
  anchor = checkButton

  checkButton = CreateFrame('CheckButton', 'rarityCheckButton', farmerOptionsFrame, 'OptionsCheckButtonTemplate')
  checkButton:SetPoint('TOPLEFT', anchor, 'BOTTOMLEFT', 0, -5)
  rarityCheckButtonText:SetText('show items based on rarity')
  checkButtonList['rarity'] = checkButton
  anchor = checkButton

  slider = CreateFrame('Slider', 'minimumRaritySlider', farmerOptionsFrame, 'OptionsSliderTemplate')
  slider:SetPoint('TOPLEFT', anchor, 'BOTTOMLEFT', 10, -20)
  slider:SetOrientation('HORIZONTAL')
  slider:SetMinMaxValues(0, 8)
  slider:SetValueStep(1)
  slider:SetObeyStepOnDrag(true)
  minimumRaritySliderText:SetText('minimum rarity')
  minimumRaritySliderLow:SetText('')
  minimumRaritySliderHigh:SetText('')
  slider:SetScript('OnValueChanged', function (self, value)
    displayRarity(self.edit, value)
  end)
  sliderList['minimumRarity'] = slider
  anchor = slider
  edit = CreateFrame('EditBox', 'minimumRarityEditBox', farmerOptionsFrame)
  edit:SetAutoFocus(false)
  edit:Disable()
  edit:SetPoint('TOP', anchor, 'BOTTOM', 0, 0)
  edit:SetFontObject('ChatFontNormal')
  edit:SetHeight(20)
  edit:SetWidth(slider:GetWidth())
  edit:SetTextInsets(8, 8, 0, 0)
  edit:SetJustifyH('CENTER')
  edit:Show()
  -- edit:SetBackdrop(slider:GetBackdrop())
  -- edit:SetBackdropColor(0, 0, 0, 0.8)
  -- edit:SetBackdropBorderColor(1, 1, 1, 1)
  slider.edit = edit
  anchor = edit

  slider = CreateFrame('Slider', 'fontSizeSlider', farmerOptionsFrame, 'OptionsSliderTemplate')
  slider:SetPoint('TOPLEFT', anchor, 'BOTTOMLEFT', 0, -20)
  slider:SetOrientation('HORIZONTAL')
  slider:SetMinMaxValues(8, 64)
  slider:SetValueStep(1)
  slider:SetObeyStepOnDrag(true)
  fontSizeSliderText:SetText('font size')
  fontSizeSliderLow:SetText('8')
  fontSizeSliderHigh:SetText('64')
  slider:SetScript('OnValueChanged', function (self, value)
    self.edit:SetText(value)
    self.edit:SetCursorPosition(0)
    setFontSize(value)
  end)
  sliderList['fontSize'] = slider
  anchor = slider
  edit = CreateFrame('EditBox', 'fontSizeEditBox', farmerOptionsFrame)
  edit:SetAutoFocus(false)
  edit:Disable()
  edit:SetPoint('TOP', anchor, 'BOTTOM', 0, 0)
  edit:SetFontObject('ChatFontNormal')
  edit:SetHeight(20)
  edit:SetWidth(slider:GetWidth())
  edit:SetTextInsets(8, 8, 0, 0)
  edit:SetJustifyH('CENTER')
  edit:Show()
  -- edit:SetBackdrop(slider:GetBackdrop())
  -- edit:SetBackdropColor(0, 0, 0, 0.8)
  -- edit:SetBackdropBorderColor(1, 1, 1, 1)
  slider.edit = edit
  anchor = edit

  button = CreateFrame('Button', 'moveButton', farmerOptionsFrame, 'OptionsButtonTemplate')
  button:SetPoint('BOTTOMLEFT', farmerOptionsFrame, 'BOTTOMLEFT', 10, 12)
  button:SetWidth(150)
  button:SetHeight(25)
  button:SetText('move frame')
  button:SetScript('OnClick', function (self)
    moveFrame()
  end)
  anchor = button

  button = CreateFrame('Button', 'resetPositionButton', farmerOptionsFrame, 'OptionsButtonTemplate')
  button:SetPoint('LEFT', anchor, 'RIGHT', 10, 0)
  button:SetWidth(150)
  button:SetHeight(25)
  button:SetText('reset position')
  button:SetScript('OnClick', function (self)
    setDefaultPosition()
    storePosition(farmerVars.frame)
  end)
  anchor = button
end

local function loadOptions ()
  for k, v in pairs(checkButtonList) do
    v:SetChecked(farmerOptions[k])
  end
  for k, v in pairs(sliderList) do
    v:SetValue(farmerOptions[k])
  end
end

local function saveOptions ()
  for k, v in pairs(checkButtonList) do
    farmerOptions[k] = v:GetChecked()
  end
  for k, v in pairs(sliderList) do
    farmerOptions[k] = v:GetValue()
  end
end

farmerOptionsFrame.okay = saveOptions
farmerOptionsFrame.refresh = loadOptions

function events:ADDON_LOADED (addon)
  if (addon == 'Farmer') then
    initPanel()

    if (farmerOptions == nil) then
      farmerOptions = {}
      farmerOptions.version = currentVersion
    end

    if (farmerOptions.version == nil or
        farmerOptions.version < currentVersion) then
      farmerOptions.version = currentVersion
      print('You can let Farmer display Money now! \nCan be enabled in the options')
    end

    if (farmerOptions.fastLoot == nil) then
      farmerOptions.fastLoot = true
    end

    if (farmerOptions.rarity == nil) then
      farmerOptions.rarity = true
    end

    if (farmerOptions.minimumRarity == nil) then
      farmerOptions.minimumRarity = 3
    end

    if (farmerOptions.currency == nil) then
      farmerOptions.currency = true
    end

    if (farmerOptions.reagents == nil) then
      farmerOptions.reagents = true
    end

    if (farmerOptions.money == nil) then
      farmerOptions.money = false
    end

    if (farmerOptions.special == nil) then
      farmerOptions.special = true
    end

    if (farmerOptions.fontSize == nil) then
      farmerOptions.fontSize = 24
    end
    setFontSize(farmerOptions.fontSize)

    if (farmerOptions.anchor == nil) then
      setDefaultPosition()
    else
      farmerVars.frame:SetPoint(unpack(farmerOptions.anchor))
    end
  end
end

function events:PLAYER_LOGIN ()
  if (earningStamp == nil) then
    earningStamp = GetMoney()
  end

  if (farmerOptions.money == true) then
    farmerVars.moneyStamp = GetMoney()
  end

  LootFrame:SetAlpha(0)
end

local function eventHandler (self, event, ...)
  events[event](self, ...)
end

farmerOptionsFrame:SetScript('OnEvent', eventHandler)

for k, v in pairs(events) do
  farmerOptionsFrame:RegisterEvent(k)
end

--[[
///#############################################################################
/// slash commands
///#############################################################################
--]]

local slashCommands = {}

function slashCommands:move ()
  moveFrame()
end

function slashCommands:reset ()
  setDefaultPosition()
  storePosition(farmerVars.frame)
end

function slashCommands:gold (param)
  if (param == 'reset') then
    earningStamp = GetMoney()
    print('Money counter was reset')
    return
  end
  local difference = GetMoney() - earningStamp
  local amount = math.abs(difference)
  local text

  text = (GOLD_AMOUNT_TEXTURE .. ' ' ..
          SILVER_AMOUNT_TEXTURE .. ' ' ..
          COPPER_AMOUNT_TEXTURE):format(amount / 10000, 0, 0,
                                        (amount / 100) % 100, 0, 0,
                                        amount % 100, 0, 0)
  if (difference >= 0) then
    print('Money earned this session: ' .. text)
  else
    print('You lost money this session: ' .. text)
  end
end

function slashCommands:default ()
  InterfaceOptionsFrame_Show()
  InterfaceOptionsFrame_OpenToCategory(farmerOptionsFrame)
end

local function slashHandler (input)
  local command = string.match(input, '^([^ ]+)')
  local _, param = string.match(input, '^([^ ]+) ([^ ]*)')
  if (command ~= nil) then
    command = string.lower(command)
  else
    command = 'default'
  end

  if (param ~= nil) then
    param = string.lower(param)
  end

  if (slashCommands[command] ~= nil) then
    slashCommands[command](nil, param)
    return
  end
  print('Farmer: unknown command "' .. input .. '"')
end

SLASH_FARMER1 = '/farmer'
SlashCmdList['FARMER'] = slashHandler
